SystemOrganization addCategory: #'Seaside-Adaptors-Comanche'!

WAServerAdaptor subclass: #WAComancheRequestConverter
	instanceVariableNames: 'streamUploads'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Comanche'!

WAComancheRequestConverter class
	instanceVariableNames: 'default'!

!WAComancheRequestConverter commentStamp: 'pmm 4/13/2008 20:18' prior: 0!
I am the common superclass for all classes that provided connectivity with the Comanche web server (Kom). Among other thing I do encoding and decoding of requests and responses. I have two main subclasses, WAKom is a traditional server adapter whereas WAListener is a streaming server adapter that proves Comet functioanlity. When in doubt, pick WAKom.

The encoding can be configured with the #endocing: method. The following values are allowed arguments:
nil
	switches off all encoding
	Strings will be delivered to you in the encoding the client used as ByteStrings. You can configure the encoding of these strings with the #charSet setting of your application. All Strings you return must have this encoding as well.
	
'utf-8'
	uses the utf-8 fast path by Andreas Raab
	Strings will be delivered to you as Squeak encoded Strings (minus the leading char). All the Strings you return must as well be Squeak encoded. Responses send to the client will be utf-8 encoded. You must additionally set the #charSet setting of your application to 'utf-8' (should be the default).
	
any other encoding supported by Squeak, see TextConverter allEncodingNames
	uses whatever TextConverter Squeak uses for this encoding
	Strings will be delivered to you as Squeak encoded Strings (minus the leading character). All the Strings you return must as well be Squeak encoded. Responses send to the client will be in the specified encoding. You must additionally set the #charSet setting of your application to the desired value.!

WAComancheRequestConverter class
	instanceVariableNames: 'default'!

!WAComancheRequestConverter class methodsFor: 'compatibility' stamp: 'jf 10/30/2008 16:26'!
default
	^ default! !

!WAComancheRequestConverter class methodsFor: 'compatibility' stamp: 'jf 10/30/2008 16:32'!
port
	^ default ifNotNil: [ default port ]! !

!WAComancheRequestConverter class methodsFor: 'compatibility' stamp: 'jf 10/30/2008 16:31'!
startOn: aNumber
	(default notNil and: [ default manager notNil ])
		ifFalse: [ default := self port: aNumber ]
		ifTrue: [
			default stop.
			default port: aNumber ].

	default start! !

!WAComancheRequestConverter class methodsFor: 'compatibility' stamp: 'jf 10/30/2008 16:29'!
stop
	default ifNotNil: [ default stop ]! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 4/18/2008 06:57'!
convertMultipartFileField: aChunk
	^(self fileClass fromChunk: aChunk)
		fileName: (self decodeUrl: aChunk fileName);
		yourself! !

!WAComancheRequestConverter methodsFor: 'converting' stamp: 'pmm 9/11/2008 19:49'!
convertRequest: aKomRequest
	| request |
	aKomRequest multiValueFormFieldsEnable.
	aKomRequest isPostMultipart ifTrue: [
		self processMultipartFields: aKomRequest ].

	request := WARequest
		method: aKomRequest method
		url: (self pathFrom: aKomRequest)
		headers: aKomRequest header
		fields: (self fieldsOf: aKomRequest)
		cookies: (self cookiesFrom: aKomRequest)
		remoteAddress: (self remoteAddressOf: aKomRequest)
		nativeRequest: aKomRequest.

	"needed only by SqueakSource"
	aKomRequest method = 'PUT' ifTrue: [
		request fields
			at: 'PUTData'
			put: (aKomRequest stream next: aKomRequest contentLength) ].
	^ request! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 7/11/2008 18:57'!
cookiesFrom: aKomRequest
	"directly access header fiedls to work around nave Kom cookie handling"
	^WARequestCookie fromString: (aKomRequest header
		at: 'cookie'
		ifAbsent: [''] )! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'jf 9/11/2008 13:24'!
decodeFields: aDictionary using: aOneArgumentBlock into: aDictionaryMerger
	aDictionary keysAndValuesDo: [ :key :value |
		| decoded |
		decoded := (value isString)
			ifTrue: [ aOneArgumentBlock value: value ]
			ifFalse: [
				(value isCollection)
					ifTrue: [ value collect: [ :string |
						aOneArgumentBlock value: string ] ]
					ifFalse: [ value ] ].
		aDictionaryMerger at: key put: decoded ]! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 9/4/2008 14:37'!
fieldsOf: aKomRequest
	"returns all the GET and POST parameters of the given request decoded"
	| merger |
	merger := WADictionaryMerger new.
	self getFieldsOf: aKomRequest into: merger.
	self postFieldsOf: aKomRequest into: merger.
	^merger dictionary! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 4/18/2008 06:54'!
fileClass
	^streamUploads
		ifTrue: [ WATemporaryFile ]
		ifFalse: [ WAFile ]! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 9/1/2008 13:16'!
flatten: aDictionary
	^(aDictionary isKindOf: HttpFormDictionary)
		ifTrue: [
			| result |
			result := Dictionary new.
			aDictionary keysAndValuesDo: [ :key :value |
				value size = 1
					ifTrue: [ result at: key put: value first  ]
					ifFalse: [ result at: key put: value ] ].
			result ]
		ifFalse: [ aDictionary copy ]! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 9/4/2008 14:36'!
getFieldsOf: aKomRequest into: aDictionarMerger
	"puts all the GET parameters of the given request decoded into a dictionary merger"
	^self
		decodeFields: aKomRequest getFields
		using: [ :each | self decodeUrl: each ]
		into: aDictionarMerger! !

!WAComancheRequestConverter methodsFor: 'initialization' stamp: 'jf 10/29/2008 18:40'!
initialize
	super initialize.
	streamUploads := false! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 7/11/2008 19:00'!
pathFrom: aKomRequest
	^ String streamContents: [:stream |
		(self pathSegmentsFrom: aKomRequest)
			do:	[ :each |
				stream nextPut: $/.
				stream nextPutAll: each ] ]! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 7/11/2008 18:59'!
pathSegmentsFrom: aKomRequest
	"We need to use #rawUrl because Comanche has already unescaped #url without
	splitting on /. This means that we can't identify the difference between a real /
	and a %2f that might have been in the original URL."

	^ ((aKomRequest rawUrl copyUpTo: $?) findTokens: '/')
		collect:	[ :each |
			self decodeUrl: (each unescapePercentsWithTextEncoding: nil) ]! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 9/4/2008 14:37'!
postFieldsOf: aKomRequest into: aDictionaryMerger
	"puts all the POST parameters of the given request decoded into a dictionary merger"
	| isUrlEncoded |
	isUrlEncoded := aKomRequest contentType = 'application/x-www-form-urlencoded'.
	^self
		decodeFields: aKomRequest postFields
		using: [ :each |
			isUrlEncoded 
				ifTrue: [ self decodeUrl: each ]
				ifFalse: [ self decode: each ] ]
		into: aDictionaryMerger! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'jf 10/29/2008 15:56'!
processMultipartFields: aKomRequest
	| merger |
	merger := WADictionaryMerger new.	

	aKomRequest multipartFormFieldsDo: [ :chunk |
		| key value |
		key := chunk fieldName.
		value := chunk fileName isEmptyOrNil
			ifTrue: [
				String streamContents: [ :stream |
					chunk saveToStream: stream ] ]
			ifFalse: [ self convertMultipartFileField: chunk ].
		merger at: key put: value ].
	
	"ugly"
	aKomRequest propertyAt: #postFields put: merger dictionary! !

!WAComancheRequestConverter methodsFor: 'private' stamp: 'pmm 5/11/2008 22:18'!
remoteAddressOf: aKomRequest
	"converts a Squeak address (ByteArray) to an IPv4 String"
	| remoteAddress |
	remoteAddress := aKomRequest remoteAddress.
	remoteAddress isString ifTrue: [
		"so apparently someone has a new VM with IPv6 support"
		^remoteAddress ].
	
	^String streamContents: [ :stream |
		remoteAddress
			do: [ :each | stream print: each ]
			separatedBy: [ stream nextPut: $. ] ]! !

!WAComancheRequestConverter methodsFor: 'accessing' stamp: 'pmm 4/17/2008 20:42'!
streamUploads: aBoolean
	"enables or disables upload streaming
	if set to true, then all file uploads will be written to a temparory file"
	streamUploads := aBoolean
	! !

WAComancheRequestConverter subclass: #WAKom
	instanceVariableNames: 'service'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Comanche'!

!WAKom commentStamp: 'pmm 4/13/2008 20:15' prior: 0!
I provide an adapter between Seaside and the Comanche web server. To start a new server on port 8080, evaluate

	WAKom startOn: 8080.
	
and to stop it, evaluate

	WAKom stop.

By default I don't do any input conversion at all, you will get the input in whatever encoding the client sent and are expected to deliver it in the same. Howerver this behavior can be changed by evaluating
WAKom default encoding: anEncoding
See the class comment of the superclass for a discussion on this topic.

The state of the service (running or stopped) is automatically restored when quitting and reopening an image.

If you want to reset the password of the config application (reachable at /seaside/config) evaluate

	WADispatcherEditor initialize.!

!WAKom class methodsFor: 'testing' stamp: 'MAD 11/4/2008 16:08'!
isConcrete
	^ true! !

!WAKom methodsFor: 'private-controlling' stamp: 'jf 10/30/2008 13:28'!
basicStart
	service := self createService.
	service start! !

!WAKom methodsFor: 'private-controlling' stamp: 'jf 10/30/2008 13:29'!
basicStop
	service ifNotNil: [ service unregister ].
	service := nil! !

!WAKom methodsFor: 'private' stamp: 'pmm 5/11/2008 16:34'!
contentsOfResponse: aResponse
	| responseStream |
	responseStream := aResponse contents.
	^aResponse contentType isBinary
		ifTrue: [ responseStream ]
		ifFalse: [ (self encode: responseStream contents) readStream binary; reset ]! !

!WAKom methodsFor: 'private' stamp: 'pmm 4/5/2008 17:45'!
convertCookiesFrom: aWAResonse to: aKomResponse
	| cookies |
	cookies := aWAResonse cookies.
	cookies isEmpty ifTrue: [ ^self ].

	cookies do: [ :each |
		"user Seaside serialization for full features instead of Kom
		this is quite evil code that works only because it knows the exact implemation of this method in Kom"
		aKomResponse
			setCookieName: each key
			value: each oldNetscapeStringWithoutKey
			path: nil ].
		
	aKomResponse
		fieldAt: 'Set-Cookie2'
		put: (WACookie combine: cookies using: [ :each | each rfc2965String ])! !

!WAKom methodsFor: 'converting' stamp: 'pmm 10/27/2008 19:47'!
convertResponse: aResponse 
	| komResponse |
	aResponse isNil
		ifTrue:  [ ^ self requestHandlingAborted ].
	komResponse := HttpResponse new.
	self 
		convertCookiesFrom: aResponse
		to: komResponse.
		
	aResponse headers associationsDo:  [ :associaction | 
		komResponse 
			fieldAt: associaction key
			put: associaction value ].
	komResponse status: (HttpResponse statusSymbolFor: aResponse status).
	komResponse contents: (self contentsOfResponse: aResponse).
	komResponse contentType: aResponse contentType seasideString.
	^ komResponse! !

!WAKom methodsFor: 'private-controlling' stamp: 'jf 10/29/2008 16:36'!
createService
	^(HttpService on: self port)
		name: 'seaside';
		plug: self;
		yourself! !

!WAKom methodsFor: 'testing' stamp: 'jf 10/30/2008 10:38'!
isRunning
	^ service notNil and: [ service isRunning ]! !

!WAKom methodsFor: 'testing' stamp: 'jf 10/30/2008 10:50'!
isStarting
	^ service notNil and: [ service isStopped ]! !

!WAKom methodsFor: 'testing' stamp: 'jf 10/30/2008 10:42'!
isStopped
	^ service isNil! !

!WAKom methodsFor: 'testing' stamp: 'jf 10/30/2008 10:44'!
isStopping
	^ service notNil and: [ service isStopping ]! !

!WAKom methodsFor: 'actions' stamp: 'jf 11/4/2008 13:51'!
process: komRequest
	| request response komResponse  |
	request := self convertRequest: komRequest.
	response := self dispatchRequest: request.
	komResponse := self convertResponse: response.
	response release.
	^ komResponse! !

!WAKom methodsFor: 'kom' stamp: 'ab 5/18/2003 13:53'!
processHttp
	HttpResponse current: (self process: HttpRequest current).
	^ true! !

!WAKom methodsFor: 'kom' stamp: 'ab 5/18/2003 11:24'!
processHttpRequest: aRequest
	^ self process: aRequest! !

!WAKom methodsFor: 'private' stamp: 'pmm 10/27/2008 19:47'!
requestHandlingAborted
	^ (HttpResponse fromString: 'Request handling aborted; reload to retry')
		contentType: 'text/plain';
		yourself! !

!WAKom methodsFor: 'mode' stamp: 'jf 10/29/2008 16:51'!
setDebugMode
	service setDebugMode! !

!WAKom methodsFor: 'mode' stamp: 'jf 10/29/2008 16:51'!
setDeploymentMode
	service setDeploymentMode! !

!WAKom methodsFor: 'mode' stamp: 'jf 10/29/2008 16:52'!
setVerboseMode
	service setVerboseMode! !

!WAKom methodsFor: 'system startup' stamp: 'jf 10/29/2008 18:17'!
shutDown
	service ifNotNil: [ service stop ]! !

!WAKom methodsFor: 'system startup' stamp: 'jf 10/29/2008 18:17'!
startUp
	service ifNotNil: [ service start ]! !

!WAKom methodsFor: 'printing' stamp: 'lr 11/3/2008 20:38'!
statusString
	^ (service ifNil: [ super statusString ]) asString! !

!WAKom methodsFor: 'kom' stamp: 'ab 5/18/2003 14:11'!
validateForStack: anArray! !

WAKom subclass: #WAKomEncoded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Comanche'!

!WAKomEncoded commentStamp: 'pmm 4/13/2008 20:18' prior: 0!
I provide an adapter between Seaside and the Comanche web server and by default do response/request conversion from UTF-8 to Squeak encoding (minus the leading char) and back in Squeak 3.8 and above. The same behavior can be achieved by evaluating:

WAKom default encoding: 'uft-8'

See the class comments of WAKom and WAComacheRequestConverter for more information regarding encoding.!

!WAKomEncoded methodsFor: 'defaults' stamp: 'jf 10/29/2008 17:58'!
defaultCodec
	^ SeasidePlatformSupport codecForEncoding: 'utf-8'! !

WAKomEncoded subclass: #WAKomEncoded39
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Comanche'!

!WAKomEncoded39 commentStamp: 'pmm 4/13/2008 20:20' prior: 0!
I have exactly the same resposibility as WAKomEncoded and are only provided for compatility reasons with old versions. I should not be used anymore.!

WAComancheRequestConverter subclass: #WAListener
	instanceVariableNames: 'process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Comanche'!

!WAListener commentStamp: 'pmm 4/22/2008 07:15' prior: 0!
I am a streaming server that directly writes the resulting data onto the socket. I use the request parsing of Kom, otherwise I am a web server on my own.

Note that the WAListener is not supposed to support redirects during the rendering phase, like it is done in WABasicAuthentication. It might work in the case the buffers of the socket stream haven't been flushed yet, but in most cases it doesn't. This is not a bug and will never be changed.!

!WAListener class methodsFor: 'testing' stamp: 'MAD 11/4/2008 16:09'!
isConcrete
	^ true! !

!WAListener methodsFor: 'private-controlling' stamp: 'lr 11/2/2008 15:57'!
basicStart
	process := [ [ self listenLoop ] repeat ] newProcess.
	process priority: Processor highIOPriority.
	process name: 'seaside'.
	process resume! !

!WAListener methodsFor: 'private-controlling' stamp: 'jf 10/30/2008 13:29'!
basicStop
	process ifNotNil: [ process terminate ].
	process := nil! !

!WAListener methodsFor: 'testing' stamp: 'jf 10/30/2008 10:54'!
isRunning
	^ process notNil and: [ process isTerminated not ]! !

!WAListener methodsFor: 'testing' stamp: 'jf 10/30/2008 10:59'!
isStarting
	^ process notNil and: [ process isSuspended ]! !

!WAListener methodsFor: 'testing' stamp: 'jf 10/30/2008 10:51'!
isStopped
	^ process isNil! !

!WAListener methodsFor: 'testing' stamp: 'jf 10/30/2008 10:53'!
isStopping
	^ process notNil and: [ process isTerminated ]! !

!WAListener methodsFor: 'private' stamp: 'pmm 8/31/2008 14:39'!
listenLoop
	| socket |
	socket := Socket newTCP.
	socket listenOn: port backlogSize: 50.
	socket isValid
		ifFalse: [ self error: 'Cannot create socket on port ' , port seasideString ].
	
	[ [ socket isValid ifFalse: [
			"will trigger #ifCurtailed: block and destroy socket"
		^ self listenLoop ].
		self waitForConnectionOn: socket ] repeat ] ifCurtailed: [
			"probably copy pasted from ConnectionQueue >> #listenLoop"
			(Delay forMilliseconds: 10) wait.
			socket destroy ]! !

!WAListener methodsFor: 'private' stamp: 'pmm 1/3/2008 20:36'!
outputStreamFor: aSocketStream
	^WAEncodingStream on: aSocketStream codec: self codec! !

!WAListener methodsFor: 'private' stamp: 'pmm 10/14/2006 14:19'!
processConnection: aSocket
	| stream |
	stream := SocketStream on: aSocket.
	stream autoFlush: true.
	[ [ [ self
			writeResponseForRequest: (HttpRequest readFromStream: stream)
			on: (self outputStreamFor: stream). ]
		ensure: [ stream close ] ]
		ifCurtailed: [ aSocket destroy ] ]
		forkAt: Processor userBackgroundPriority! !

!WAListener methodsFor: 'system startup' stamp: 'jf 10/29/2008 18:23'!
shutDown
	process ifNotNil: [ process terminate ]! !

!WAListener methodsFor: 'system startup' stamp: 'jf 10/29/2008 18:23'!
startUp
	process ifNotNil: [ self restart ]! !

!WAListener methodsFor: 'private' stamp: 'lr 6/28/2006 14:12'!
waitForConnectionOn: aSocket
	| connection |
	connection := (aSocket waitForAcceptFor: 10)
		ifNil: [ ^ self ].
	self processConnection: connection! !

!WAListener methodsFor: 'private' stamp: 'jf 11/4/2008 13:51'!
writeResponseForRequest: aRequest on: aStream 
	| request response |
	request := self convertRequest: aRequest.
	request responseStream: aStream.
	response := self dispatchRequest: request.
	response ifNil: [ ^ self ].
	response class = WAResponse ifTrue: 
		[ aStream resetBuffers.
		response contentType isBinary ifTrue: [ aStream binary ] ].
	response writeOn: aStream.
	response release! !

!WAPackage class methodsFor: '*Seaside-Adaptors-Comanche' stamp: 'jf 11/1/2008 11:40'!
seasideAdaptorsComanche
	^ self new
		name: 'Seaside-Adaptors-Comanche';
		addDependency: 'Seaside-Squeak-Core';
		yourself! !

Object subclass: #WAEncodingStream
	instanceVariableNames: 'stream codec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Comanche'!

!WAEncodingStream commentStamp: 'pmm 10/14/2006 14:37' prior: 0!
I wrap a SocketStream and encode the input to utf-8.
Unfortunately such hacks are needed because of the Squeak Stream hierarchy.!

!WAEncodingStream class methodsFor: 'instance creation' stamp: 'pmm 1/3/2008 20:32'!
on: aStream codec: aCodec
	^self new
		setStream: aStream codec: aCodec;
		yourself! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 1/5/2008 16:06'!
binary
	codec := WANullCodec new! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 10/14/2006 04:09'!
crlf
	stream crlf! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 10/14/2006 11:14'!
isBinary
	^false! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 1/3/2008 20:34'!
nextPut: aCharacter
	self nextPutAll: (String with: aCharacter)! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 1/3/2008 20:33'!
nextPutAll: aColllection
	stream nextPutAll: (codec encode: aColllection)! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 1/3/2008 20:41'!
resetBuffers
	^stream resetBuffers! !

!WAEncodingStream methodsFor: 'initialize-release' stamp: 'pmm 1/3/2008 20:31'!
setStream: aStream codec: aCodec
	stream := aStream.
	codec := aCodec! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 10/14/2006 11:16'!
socket
	^stream socket! !

!WAEncodingStream methodsFor: 'accessing' stamp: 'pmm 1/3/2008 20:41'!
space
	^stream space! !

!WAFile class methodsFor: '*Seaside-Adaptors-Comanche' stamp: 'pmm 4/18/2008 06:51'!
fromChunk: aChunk
	^ self new initializeFromChunk: aChunk! !

!WAFile methodsFor: '*Seaside-Adaptors-Comanche' stamp: 'lr 10/20/2008 20:54'!
initializeFromChunk: aChunk
	self contentType: aChunk contentType seasideMimeType.
	self contents: (String streamContents: [ :stream |
		aChunk saveToStream: stream ]).! !

!WAFile methodsFor: '*Seaside-Adaptors-Comanche' stamp: 'pmm 4/17/2008 20:46'!
isTemporary
	"this is not a temporary but an in memory file"
	^false! !

WAFile subclass: #WATemporaryFile
	instanceVariableNames: 'temporaryFileName fileSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Comanche'!

!WATemporaryFile commentStamp: 'pmm 4/18/2008 22:52' prior: 0!
I represent a file that was uploaded by the user via #fileInput. Compared to WAFile my contents are not stored in memory on disk in a temporary file. I support the same protocol as WAFile.

Instance Variables
	fileSize:		<Integer>
	temporaryFileName:		<String>

fileSize
	- the file size in bytes

temporaryFileName
	- the name of the file in which my contents are stored on disk!

!WATemporaryFile methodsFor: 'accessing' stamp: 'pmm 8/1/2008 14:11'!
contents
	"Reads the contents of the file into memory and returns them.
	Keep in mind that the contents will be cached into an instance variable."
	contents isNil ifTrue: [
		contents := self readStreamDo: [ :stream |
			stream contents ] ].
	^contents! !

!WATemporaryFile methodsFor: 'actions' stamp: 'pmm 8/1/2008 14:10'!
delete
	"deletes the uploaded file from the disk"
	self temporaryDirectory deleteFileNamed: self temporaryFileName! !

!WATemporaryFile methodsFor: 'initialize-release' stamp: 'pmm 11/4/2008 20:16'!
initializeFromChunk: aChunk
	"This is where we actually read the file contents chunk wise
	and save it to disk."

	| stream |
	stream := StandardFileStream newFileNamed: (self temporaryDirectory fullNameFor: self temporaryFileName).
	fileSize := [ stream binary.
		aChunk saveToStream: stream ]
			ensure: [ stream close ]! !

!WATemporaryFile methodsFor: 'testing' stamp: 'pmm 4/17/2008 20:46'!
isTemporary
	"this is not an in memory but an in temporary file"
	^true! !

!WATemporaryFile methodsFor: 'actions' stamp: 'pmm 8/1/2008 14:15'!
readStreamDo: aOneArgumentBlock
	"Convenience method to access a read stream on the contents. The read stream will be passed to the block.
	the stream will be in binary mode
	the stream will be closed afterwards"
	| stream |
	stream := self temporaryDirectory readOnlyFileNamed: self temporaryFileName.
	^[	stream binary.
		aOneArgumentBlock value: stream ]
			ensure: [ stream close ]! !

!WATemporaryFile methodsFor: 'accessing' stamp: 'pmm 4/18/2008 22:46'!
size
	"the file size in bytes"
	^fileSize! !

!WATemporaryFile methodsFor: 'accessing' stamp: 'pmm 4/18/2008 22:50'!
temporaryDirectory
	"the directory in which the file is located in which my contents are stored on the server"
	^(FileDirectory default directoryNamed: 'seasidetmp') assureExistence! !

!WATemporaryFile methodsFor: 'accessing' stamp: 'lr 8/30/2008 16:22'!
temporaryFileName
	"the name of the file in which my contents are stored in the directory #temporaryDirectory
	
	don't confuse this with #fileName which is the name of the file that the client sent"
	^ temporaryFileName ifNil: [ temporaryFileName := (WAExternalID new: 48) seasideString ]! !
